"""
Tests for the RollbarHandler logging handler
"""
import copy
import logging
import mock
import urllib

import rollbar
from rollbar.logger import RollbarHandler

from . import BaseTest


_test_access_token = 'aaaabbbbccccddddeeeeffff00001111'
_test_environment = 'test'
_default_settings = copy.deepcopy(rollbar.SETTINGS)


class LogHandlerTest(BaseTest):
    def setUp(self):
        rollbar._initialized = False
        rollbar.SETTINGS = copy.deepcopy(_default_settings)

    def _create_logger(self):
        logger = logging.getLogger(__name__)
        logger.setLevel(logging.DEBUG)

        rollbar_handler = RollbarHandler(_test_access_token, _test_environment)
        rollbar_handler.setLevel(logging.WARNING)

        logger.addHandler(rollbar_handler)

        return logger

    @mock.patch('rollbar.send_payload')
    def test_message_stays_unformatted(self, send_payload):
        logger = self._create_logger()
        logger.warning("Hello %d %s", 1, 'world')

        payload = send_payload.call_args[0][0]
        
        self.assertEqual(payload['data']['body']['message']['body'], "Hello %d %s")
        self.assertEqual(payload['data']['body']['message']['args'], (1, 'world'))
        
        self.assertEqual(payload['data']['body']['message']['record']['name'], __name__)




import difflib
import pprint
import unittest


# from http://hg.python.org/cpython/file/67ada6ab7fe2/Lib/unittest/util.py
# for Python 2.6 support
_MAX_LENGTH = 80
def safe_repr(obj, short=False):
    try:
        result = repr(obj)
    except Exception:
        result = object.__repr__(obj)
    if not short or len(result) < _MAX_LENGTH:
        return result
    return result[:_MAX_LENGTH] + ' [truncated]...'


class BaseTest(unittest.TestCase):
    # from http://hg.python.org/cpython/file/67ada6ab7fe2/Lib/unittest/case.py
    # for Python 2.6 support

    longMessage = False

    def _formatMessage(self, msg, standardMsg):
        """Honour the longMessage attribute when generating failure messages.
        If longMessage is False this means:
        * Use only an explicit message if it is provided
        * Otherwise use the standard message for the assert

        If longMessage is True:
        * Use the standard message
        * If an explicit message is provided, plus ' : ' and the explicit message
        """
        if not self.longMessage:
            return msg or standardMsg
        if msg is None:
            return standardMsg
        try:
            # don't switch to '{}' formatting in Python 2.X
            # it changes the way unicode input is handled
            return '%s : %s' % (standardMsg, msg)
        except UnicodeDecodeError:
            return  '%s : %s' % (safe_repr(standardMsg), safe_repr(msg))

    def assertIsInstance(self, obj, cls, msg=None):
        """Same as self.assertTrue(isinstance(obj, cls)), with a nicer
        default message."""
        if not isinstance(obj, cls):
            standardMsg = '%s is not an instance of %r' % (safe_repr(obj), cls)
            self.fail(self._formatMessage(msg, standardMsg))

    def assertDictEqual(self, d1, d2, msg=None):
        self.assertIsInstance(d1, dict, 'First argument is not a dictionary')
        self.assertIsInstance(d2, dict, 'Second argument is not a dictionary')

        if d1 != d2:
            standardMsg = '%s != %s' % (safe_repr(d1, True), safe_repr(d2, True))
            diff = ('\n' + '\n'.join(difflib.ndiff(
                           pprint.pformat(d1).splitlines(),
                           pprint.pformat(d2).splitlines())))
            standardMsg = self._truncateMessage(standardMsg, diff)
            self.fail(self._formatMessage(msg, standardMsg))

    def assertIn(self, member, container, msg=None):
        """Just like self.assertTrue(a in b), but with a nicer default message."""
        if member not in container:
            standardMsg = '%s not found in %s' % (safe_repr(member),
                                                  safe_repr(container))
            self.fail(self._formatMessage(msg, standardMsg))

    def assertNotIn(self, member, container, msg=None):
        """Just like self.assertTrue(a not in b), but with a nicer default message."""
        if member in container:
            standardMsg = '%s unexpectedly found in %s' % (safe_repr(member),
                                                        safe_repr(container))
            self.fail(self._formatMessage(msg, standardMsg))



"""
Tests for Flask instrumentation
"""

import json
import sys
import os

import mock
import rollbar

from rollbar.test import BaseTest

# access token for https://rollbar.com/rollbar/pyrollbar
TOKEN = '92c10f5616944b81a2e6f3c6493a0ec2'

# Flask doesn't work on python 3.2, so don't test there.
ALLOWED_PYTHON_VERSION = not (sys.version_info[0] == 3 and sys.version_info[1] == 2)


def create_app():
    from flask import Flask, Request, got_request_exception
    import rollbar.contrib.flask
    app = Flask(__name__)

    @app.route('/')
    def index():
        return 'Index page'

    @app.route('/cause_error', methods=['GET', 'POST'])
    def cause_error():
        raise Exception("Uh oh")
    
    @app.before_first_request
    def init_rollbar():
        rollbar.init(TOKEN, 'flasktest',
                     root=os.path.dirname(os.path.realpath(__file__)),
                     allow_logging_basic_config=True)
        got_request_exception.connect(rollbar.contrib.flask.report_exception, app)

    class CustomRequest(Request):
        @property
        def rollbar_person(self):
            return {'id': '123', 'username': 'testuser', 'email': 'test@example.com'}

    app.request_class = CustomRequest
    
    return app


class FlaskTest(BaseTest):
    def setUp(self):
        super(FlaskTest, self).setUp()
        if not ALLOWED_PYTHON_VERSION:
            return
        self.app = create_app()
        self.client = self.app.test_client()
        
    def test_index(self):
        if not ALLOWED_PYTHON_VERSION:
            return
        
        resp = self.client.get('/')
        self.assertEqual(resp.status_code, 200)

    def assertStringEqual(self, left, right):
        if sys.version_info[0] > 2:
            if hasattr(left, 'decode'):
                left = left.decode('ascii')
            if hasattr(right, 'decode'):
                right = right.decode('ascii')

            return self.assertEqual(left, right)
        else:
            return self.assertEqual(left, right)

    @mock.patch('rollbar.send_payload')
    def test_uncaught(self, send_payload):
        if not ALLOWED_PYTHON_VERSION:
            return
        
        resp = self.client.get('/cause_error?foo=bar',
            headers={'X-Real-Ip': '1.2.3.4', 'User-Agent': 'Flask Test'})
        self.assertEquals(resp.status_code, 500)

        self.assertEqual(send_payload.called, True)
        payload = send_payload.call_args[0][0]
        data = payload['data']

        self.assertIn('body', data)
        self.assertEqual(data['body']['trace']['exception']['class'], 'Exception')
        self.assertStringEqual(data['body']['trace']['exception']['message'], 'Uh oh')
        
        self.assertIn('person', data)
        self.assertDictEqual(data['person'],
            {'id': '123', 'username': 'testuser', 'email': 'test@example.com'})

        self.assertIn('request', data)
        self.assertEqual(data['request']['url'], 'http://localhost/cause_error?foo=bar')
        self.assertDictEqual(data['request']['GET'], {'foo': ['bar']})
        self.assertEqual(data['request']['user_ip'], '1.2.3.4')
        self.assertEqual(data['request']['method'], 'GET')
        self.assertEqual(data['request']['headers']['User-Agent'], 'Flask Test')

    @mock.patch('rollbar.send_payload')
    def test_uncaught_json_request(self, send_payload):
        if not ALLOWED_PYTHON_VERSION:
            return
        
        json_body = json.dumps({"hello": "world"})
        resp = self.client.post('/cause_error', data=json_body,
            headers={'Content-Type': 'application/json', 'X-Forwarded-For': '5.6.7.8'})
        
        self.assertEquals(resp.status_code, 500)

        self.assertEqual(send_payload.called, True)
        payload = send_payload.call_args[0][0]
        data = payload['data']

        self.assertIn('body', data)
        self.assertEqual(data['body']['trace']['exception']['class'], 'Exception')
        self.assertStringEqual(data['body']['trace']['exception']['message'], 'Uh oh')
        
        self.assertIn('person', data)
        self.assertDictEqual(data['person'],
            {'id': '123', 'username': 'testuser', 'email': 'test@example.com'})

        self.assertIn('request', data)
        self.assertEqual(data['request']['url'], 'http://localhost/cause_error')
        self.assertEqual(data['request']['body'], json_body)
        self.assertEqual(data['request']['user_ip'], '5.6.7.8')
        self.assertEqual(data['request']['method'], 'POST')

